#ifndef FRAGMENTS_H
#define FRAGMENTS_H
/*
 *  fragments.h
 *  TopHat
 *
 *  Created by Cole Trapnell on 1/14/09.
 *  Copyright 2009 Cole Trapnell. All rights reserved.
 *
 */

#include "bwt_map.h"

typedef BowtieHit FragmentAlignment;

struct FragmentAlignmentGrade
{
	FragmentAlignmentGrade() : 
	status(UNALIGNED),
	edit_dist(0) {}
	
	FragmentAlignmentGrade(const BowtieHit& h1) 
	{
		if (!h1.contiguous())
		{
			status = SPLICED;
		}
		else
		{
			status = CONTIGUOUS;
		}
		edit_dist = h1.edit_dist(); 
	}
	
	FragmentAlignmentGrade& operator=(const FragmentAlignmentGrade& rhs)
	{
		status = rhs.status;
		edit_dist = edit_dist;
		return *this;
	}
	
	// Returns true if rhs is a "happier" alignment for the ends of this insert
	// than this InsertStatus.
	bool operator<(const FragmentAlignmentGrade& rhs)
	{
		if (status != rhs.status)
			return status < rhs.status;
		
		return rhs.edit_dist < edit_dist;
	}
	
	uint8_t status;
	uint8_t edit_dist;
};

typedef vector<pair<FragmentAlignmentGrade, vector<FragmentAlignment*> > > BestFragmentAlignmentTable;

void best_fragment_mappings(uint64_t refid,
							const string& name,
							HitList& hits1_in_ref,
							ReadTable& it,
							BestFragmentAlignmentTable& best_status_for_fragments);

void accept_best_hits(BestFragmentAlignmentTable& best_status_for_fragments);
void accept_unique_hits(BestFragmentAlignmentTable& best_status_for_fragments);

#endif
